#include "framework.h"
#include "chemistry factory.h"
#include <gl/gl.h>
#include <stdio.h>
#include <math.h>
#include <vector>
using namespace std;
#include "CFont.h"
#include "Cmenu.h"
#include "CBase.h"
#include "Cconnect.h"


constexpr float Field = 30.0F;

CBase::CBase(HWND _hWnd)
{
    isInicjalized = false;
    this->hWnd = _hWnd;
    posX = 0.01375F;
    posY = 0.015556F;
    real_posX = 0.42F;
    real_posY = 0.46F;
    zoom = 1;
    old_xPos = 0;
    old_yPos = 0;
    xPos = 0;
    yPos = 0;
    tree = new CmapGen();
    menu = new Cmenu();
    connect = new Cconnect(this);
    font = new CFont(this);
    state_opengl = -1;
    line_with = -1;
    Selected = 0;
    for (int i = 0; i < 48; i++)
    {
        double ii = (double)i;
        sinus[i] = (float)sin(((double)(ii * 8.0F)) * (3.14F / 180.0f));
        cosinus[i] = (float)cos(((double)(ii * 8.0F)) * (3.14F / 180.0f));
    }
    Init();
    timer = GetTickCount64();
    timer2 = GetTickCount64();
}

CBase::~CBase()
{
    if (tree != NULL)
        delete tree;
    tree = NULL;
    if (!isInicjalized)
        return;
    isInicjalized = false;
    /* shutdown OpenGL */
    DisableOpenGL(hWnd, hDC, hRC);
}

//openGL init
bool CBase::Init()
{
    /* enable OpenGL for the window */
    EnableOpenGL(hWnd, &hDC, &hRC);
    GetDesktopResolution(resx, resy);
    isInicjalized = menu->Init(resx, resy, this);
    tree->CreateMap();
    misja.lvl = 1;
    misja.object_Count = 10;
    misja.object_type = CSixTree::Field::_sub_type::Blue;
    float object_rand = tree->GetRandom();
    if (object_rand < 0.5F)
        misja.object_type = CSixTree::Field::_sub_type::Triangle;
    else
        misja.object_type = CSixTree::Field::_sub_type::Square;
    if (misja.lvl > 10)
    {
        float color_rand = tree->GetRandom() * 11;
        if ((int)color_rand == misja.object_color)
            color_rand = 11;
        misja.object_color = color_rand;
    }
    else
    {
        int color_rand = (int)(tree->GetRandom() * 3);
        if (color_rand == 0)
            misja.object_color = 0;//YELLOW
        if (color_rand == 1)
            misja.object_color = 4;//BLUE
        if (color_rand == 2)
            misja.object_color = 8;//RED
    }
    return isInicjalized;
}

bool CBase::DoIntersect(float x, float y, float sizex, float sizey, float xm, float ym) {//0 fail 1,2,3,4 top,bottom,left,right

    if (x > xm)
        return false;
    if (y > ym)
        return false;
    if (x + sizex < xm)
        return false;
    if (y + sizey < ym)
        return false;
    return true;
}

bool CBase::Render()
{
    if (!isInicjalized)
        return false;
    DWORD val = GetTickCount64() - timer;
    float time_pass = ((float)val) / 1000.0F;

    timer = GetTickCount64();

    glClearColor(0.6f, 0.6f, 0.6f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT);

    glPushMatrix();

    float field_zoom = Field * zoom;
    float w, e;
    w = (float)resx;
    w /= field_zoom;
    e = (float)resy;
    e /= field_zoom;

    float fx, fy;
    fx = (1.0f / w);
    fy = (1.0f / e);
    glBegin(GL_QUADS);
    float _1_resX = (1.0F / (float)resx);
    float _1_resY = (1.0F / (float)resy);
    //narysuj tlo
    for (int x = -1; x <= w + 1; x++)
        for (int y = -1; y <= e + 1; y++)
        {
            float a, b, c, v;
            c = (_1_resX * field_zoom) / 15;
            if (c < _1_resX * 1)
                c = _1_resX * 1;
            v = (_1_resY * field_zoom) / 15;
            if (v < _1_resY * 1)
                v = _1_resY * 1;
            a = ((float)x);
            b = ((float)y);
            a /= w;
            b /= e;
            DrawBox(a + c / 2 + posX, b + v / 2 + posY, fx - c, fy - v, 0.7F, 0.7F, 0.7F);
        }
    //mysz

    float a, b, c, v;
    float xx = (xPos - posX * (float)resx) / (field_zoom);
    int x = (int)xx;
    if (xx < 0)
        x = -1;
    float yy = (yPos - posY * (float)resy) / (field_zoom);
    int y = (int)(yy);
    if (yy < 0)
        y = -1;
    c = _1_resX;
    v = _1_resY;
    a = ((float)x) / w;
    b = ((float)y) / e;
    float olda, oldb;
    olda = a;
    oldb = b;
    float xmysz = a + posX + fx;
    float ymysz = b + posY + fy;
    //wskarznik myszy
    DrawBox(a + c + posX, b + v + posY, fx - c, fy - v, 0.7f, 0, 0);
    DrawBox(a + c * 4 + posX, b + v * 4 + posY, fx - c * 7, fy - v * 7, 0.7F, 0.7F, 0.7F);

    Selected = 0;
    Selected_all = 0;
    //detect rederable objects:

    vector<CSixTree::Field*>* temp = tree->GetArea((-real_posX * resx) / (field_zoom)+(resx / (field_zoom) / 2), (-real_posY * resy) / (field_zoom)+(resy / (field_zoom) / 2), resx / (field_zoom)+3, resy / (field_zoom)+3);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        x = field->posx - (int)((posX * (float)resx) / (field_zoom));
        y = field->posy - (int)((posY * (float)resy) / (field_zoom));
        a = ((float)x) / w;
        b = ((float)y) / e;
        object_X = a + c + real_posX;
        object_Y = b + v + real_posY;
        object_sX = fx * field->sizex;
        object_sY = fy * field->sizey;
        object_czoom = c * zoom;
        object_vzoom = v * zoom;
        _r = _g = _b = 0;
        bool colizja = false;
        if (DoIntersect(object_X, object_Y, object_sX, object_sY, xmysz, ymysz))
        {
            colizja = true;
            for (size_t j = 0; j < temp->size(); j++)
            {
                CSixTree::Field* field2 = temp->at(j);
                if (field2->type == CSixTree::Field::_type::Bulding)
                    if ((field2->posx == field->posx) && (field2->posy == field->posy))
                        colizja = false;
            }

            switch (menu->button_actual_select)
            {
            case 0:
                break;
            case 1:
                break;
            default:
                DrawBox(olda + c * 4 + posX, oldb + v * 4 + posY, fx - c * 7, fy - v * 7, 0.1F, 0.7F, 0.1F);
            }
            if (field->type == CSixTree::Field::_type::Resource)
            {
                if (colizja == true)
                    Selected = field;
                else
                    Selected = 0;
            }
            Selected_all = field;
        }
        int kiers;
        switch (field->sub_type)
        {
        case CSixTree::Field::_sub_type::Blue:
            _r = connect->ColorTable[4].r;
            _g = connect->ColorTable[4].g;
            _b = connect->ColorTable[4].b;
            DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
            break;
        case CSixTree::Field::_sub_type::Red:
            _r = connect->ColorTable[8].r;
            _g = connect->ColorTable[8].g;
            _b = connect->ColorTable[8].b;
            DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
            break;
        case CSixTree::Field::_sub_type::Yellow:
            _r = connect->ColorTable[0].r;
            _g = connect->ColorTable[0].g;
            _b = connect->ColorTable[0].b;
            DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
            break;
        case CSixTree::Field::_sub_type::Under_belt_input:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = _g = _b = 0.9f;
            if ((field->kierunek == CSixTree::Field::_kierunek::right) || (field->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((field->kierunek == CSixTree::Field::_kierunek::up) || (field->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            _r = _g = 0.7f;
            _b = 0.5F;
            if (field->kierunek == CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y, object_sX, object_sY / 2, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::right)
                DrawBox(object_X + object_sX / 2, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y + object_sY / 2, object_sX, object_sY / 2, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::left)
                DrawBox(object_X, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            DrawArrow(field->kierunek);
            break;
        case CSixTree::Field::_sub_type::Under_belt_output:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = _g = _b = 0.9f;
            if ((field->kierunek == CSixTree::Field::_kierunek::right) || (field->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((field->kierunek == CSixTree::Field::_kierunek::up) || (field->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            _r = _g = 0.7f;
            _b = 0.5F;
            if (field->kierunek == CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y, object_sX, object_sY / 2, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::left)
                DrawBox(object_X + object_sX / 2, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y + object_sY / 2, object_sX, object_sY / 2, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::right)
                DrawBox(object_X, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            DrawArrow(field->kierunek);
            break;
        case CSixTree::Field::_sub_type::Paint_Mixer:
            _r = _b = _g = 0.1f;
            if ((field->kierunek == CSixTree::Field::_kierunek::right) || (field->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawTriangleFull(object_X, object_Y, object_X + object_sX / 2, object_Y, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y, object_X + object_sX, object_Y, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X, object_Y + object_sY, object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((field->kierunek == CSixTree::Field::_kierunek::up) || (field->kierunek == CSixTree::Field::_kierunek::bottom))
            {
                DrawTriangleFull(object_X, object_Y, object_X, object_Y + object_sY / 2, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X, object_Y + object_sY / 2, object_X, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX, object_Y, object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            if (field->kierunek == CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::right)
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
            if (field->kierunek == CSixTree::Field::_kierunek::left)
                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
            DrawArrow(field->kierunek);
            break;
        case CSixTree::Field::_sub_type::Output:
            _r = _g = 0.7f;
            _b = 0.1f;

            DrawBox(object_X + object_czoom, object_Y + object_vzoom, object_sX - object_czoom * 2, object_sY - object_vzoom * 2, _r, _g, _b);
            _r = _g = _b = 0.2f;
            DrawTriangleFull(object_X + object_czoom, object_Y + object_vzoom, object_X + object_sX / 3, object_Y + object_vzoom, object_X + object_czoom, object_Y + object_sY / 3, _r, _g, _b);
            DrawTriangleFull(object_X - object_czoom + object_sX, object_Y + object_vzoom, object_X + object_sX - object_sX / 3, object_Y + object_vzoom, object_X - object_czoom + object_sX, object_Y + object_sY / 3, _r, _g, _b);
            DrawTriangleFull(object_X + object_czoom, object_Y + object_sY - object_sY / 3, object_X + object_czoom, object_Y - object_vzoom + object_sY, object_X + object_sX / 3, object_Y - object_vzoom + object_sY, _r, _g, _b);
            DrawTriangleFull(object_X - object_czoom + object_sX, object_Y - object_vzoom + object_sY, object_X - object_czoom + object_sX, object_Y + object_sY - object_sY / 3, object_X + object_sX - object_sX / 3, object_Y - object_vzoom + object_sY, _r, _g, _b);
            break;
        case CSixTree::Field::_sub_type::Belt:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = _g = _b = 0.9f;
            if ((field->kierunek == CSixTree::Field::_kierunek::right) || (field->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((field->kierunek == CSixTree::Field::_kierunek::up) || (field->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            DrawArrow(field->kierunek);
            break;
        case CSixTree::Field::_sub_type::Spliter:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = 0.75f; _b = 0.75f; _g = 0.75f;
            if (field->kierunek != CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);//up
            if (field->kierunek != CSixTree::Field::_kierunek::right)
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);//left
            if (field->kierunek != CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);//down
            if (field->kierunek != CSixTree::Field::_kierunek::left)
                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);//prawy
            DrawArrow(field->kierunek);
            kiers = field->kierunek;
            kiers++;
            if (kiers > 3)
                kiers = 0;
            DrawArrow(CSixTree::Field::_kierunek(kiers));
            kiers = field->kierunek;
            kiers--;
            if (kiers < 0)
                kiers = 3;
            DrawArrow(CSixTree::Field::_kierunek(kiers));
            break;
        case CSixTree::Field::_sub_type::Miner:
            _r = _b = 0.4f;
            _g = 0.8f;
            DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
            DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
            DrawArrow(field->kierunek);
            break;
        case CSixTree::Field::_sub_type::Painter:
            _r = _b = _g = 0.1f;
            if ((field->kierunek == CSixTree::Field::_kierunek::right) || (field->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((field->kierunek == CSixTree::Field::_kierunek::up) || (field->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            if (field->kierunek == CSixTree::Field::_kierunek::up)//ok
            {
                DrawTriangleFull(object_X, object_Y, object_X, object_Y + object_sY / 2, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X, object_Y + object_sY / 2, object_X, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
            }
            if (field->kierunek == CSixTree::Field::_kierunek::right)//ok
            {
                DrawTriangleFull(object_X, object_Y, object_X + object_sX / 2, object_Y, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y, object_X + object_sX, object_Y, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
            }
            if (field->kierunek == CSixTree::Field::_kierunek::bottom)
            {
                DrawTriangleFull(object_X + object_sX, object_Y, object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
            }
            if (field->kierunek == CSixTree::Field::_kierunek::left)
            {
                DrawTriangleFull(object_X, object_Y + object_sY, object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
            }
            DrawArrow(field->kierunek);
            break;
        case CSixTree::Field::_sub_type::Square:
            _r = _g = _b = 0.4f;
            DrawBox(object_X + object_czoom, object_Y + v * zoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
            break;
        case CSixTree::Field::_sub_type::Triangle:
            _r = _g = _b = 0.4f;
            DrawTriangle(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
            break;
        }

        if (colizja == true)
        {
            switch (menu->button_actual_select)
            {
            case 1:
                if (field->type == CSixTree::Field::_type::Resource)
                {
                    _r = _b = 0.4f;
                    _g = 0.8f;
                    DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                    DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
                    DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
                    DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                    DrawArrow(menu->kierunek);
                }
                break;
            }
        }
    }
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        x = field->posx - (int)((posX * (float)resx) / (field_zoom));
        y = field->posy - (int)((posY * (float)resy) / (field_zoom));
        a = ((float)x) / w;
        b = ((float)y) / e;

        object_X = a + c + real_posX;
        object_Y = b + v + real_posY;
        object_sX = fx * field->sizex;
        object_sY = fy * field->sizey;
        object_czoom = c * zoom;
        object_vzoom = v * zoom;

        if (field->sub_type == CSixTree::Field::_sub_type::Under_belt_input)
        {
            if (field->belt_object_color != -1)
            {
                _r = connect->ColorTable[field->belt_object_color].r;
                _g = connect->ColorTable[field->belt_object_color].g;
                _b = connect->ColorTable[field->belt_object_color].b;
            }
            else
                _r = _g = _b = 0.4F;
            //0-up 1 right 2 bottom 3 right
            switch (field->belt_objtype)
            {
            case CSixTree::Field::_sub_type::Square:
                DrawBox(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Triangle:
                DrawTriangle(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Red:
                DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Yellow:
                DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Blue:
                DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
                break;
            }
        }

        //field->belt_pos
        //0-up 1 right 2 bottom 3 left
        CSixTree::Field::_kierunek kier = field->kierunek;
        if (field->belt_next.size() > 0)
        {
            if (field->belt_next.at(field->belt_next_pos)->sub_type != CSixTree::Field::_sub_type::Output)
            {
                int posx = field->belt_next.at(field->belt_next_pos)->posx;
                int posy = field->belt_next.at(field->belt_next_pos)->posy;
                posx -= field->posx;
                posy -= field->posy;
                if (posx < 0)
                    kier = CSixTree::Field::_kierunek::left;
                if (posx > 0)
                    kier = CSixTree::Field::_kierunek::right;
                if (posy < 0)
                    kier = CSixTree::Field::_kierunek::up;
                if (posy > 0)
                    kier = CSixTree::Field::_kierunek::bottom;
            }
        }
        if (kier == CSixTree::Field::_kierunek::up)
            object_Y -= field->belt_pos * object_sY;
        if (kier == CSixTree::Field::_kierunek::right)
            object_X += field->belt_pos * object_sX;
        if (kier == CSixTree::Field::_kierunek::bottom)
            object_Y += field->belt_pos * object_sY;
        if (kier == CSixTree::Field::_kierunek::left)
            object_X -= field->belt_pos * object_sX;

        if (kier == CSixTree::Field::_kierunek::up)
            b -= field->belt_pos * object_sY;
        if (kier == CSixTree::Field::_kierunek::right)
            a += field->belt_pos * object_sX;
        if (kier == CSixTree::Field::_kierunek::bottom)
            b += field->belt_pos * object_sY;
        if (kier == CSixTree::Field::_kierunek::left)
            a -= field->belt_pos * object_sX;
        _r = _g = _b = 0;
        if ((field->sub_type == CSixTree::Field::_sub_type::Belt) || (field->sub_type == CSixTree::Field::_sub_type::Spliter) || (field->sub_type == CSixTree::Field::_sub_type::Under_belt_output))
        {
            if (field->belt_object_color != -1)
            {
                _r = connect->ColorTable[field->belt_object_color].r;
                _g = connect->ColorTable[field->belt_object_color].g;
                _b = connect->ColorTable[field->belt_object_color].b;
            }
            else
                _r = _g = _b = 0.4F;
            //0-up 1 right 2 bottom 3 right
            switch (field->belt_objtype)
            {
            case CSixTree::Field::_sub_type::Square:
                DrawBox(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Triangle:
                DrawTriangle(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Red:
                DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Yellow:
                DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Blue:
                DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
                break;
            }
        }
        if (((field->sub_type == CSixTree::Field::_sub_type::Painter) || (field->sub_type == CSixTree::Field::_sub_type::Paint_Mixer)) && (field->paint.size() > 0))
        {
            if (field->belt_object_color != -1)
            {
                _r = connect->ColorTable[field->belt_object_color].r;
                _g = connect->ColorTable[field->belt_object_color].g;
                _b = connect->ColorTable[field->belt_object_color].b;
            }
            else
                _r = _g = _b = 0.4F;

            float r = connect->ColorTable[field->paint.at(0)->Paint_num].r - _r;
            float g = connect->ColorTable[field->paint.at(0)->Paint_num].g - _g;
            float sb = connect->ColorTable[field->paint.at(0)->Paint_num].b - _b;
            _r = r * field->belt_pos + _r;
            _g = g * field->belt_pos + _g;
            _b = sb * field->belt_pos + _b;
            //0-up 1 right 2 bottom 3 right
            if (field->sub_type == CSixTree::Field::_sub_type::Paint_Mixer)
            {
                _r = connect->ColorTable[field->belt_object_color].r;
                _g = connect->ColorTable[field->belt_object_color].g;
                _b = connect->ColorTable[field->belt_object_color].b;
                DrawCircle(a + real_posX + fx / 2, b + real_posY + fy / 2, fx / 2 - object_czoom * 1, fy / 2 - object_vzoom * 1, _r, _g, _b);
            }
            switch (field->Paint_objtype)
            {
            case CSixTree::Field::_sub_type::Square:
                DrawBox(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
                break;
            case CSixTree::Field::_sub_type::Triangle:
                DrawTriangle(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
                break;
            }
        }
    }
    //logika:
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Belt);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        if (field->belt_objtype != CSixTree::Field::_sub_type::None)
            if (field->belt_next.size() > 0)
                if ((field->belt_pos <= field->belt_next.at(0)->belt_pos) || (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None))
                {
                    bool targeted_started = false;
                    for (int c = 0; c < field->belt_next.at(0)->Targeted.size(); c++)
                    {
                        if (field->belt_next.at(0)->Targeted.at(c) == field)
                            targeted_started = true;
                    }
                    if (targeted_started || (field->belt_next.at(0)->Targeted.size() == 0))
                    {
                        if ((field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Output) && (field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Painter) && (field->belt_next.at(0)->Targeted.size() == 0))
                            if (!targeted_started)
                                field->belt_next.at(0)->Targeted.push_back(field);//zaznaczenie celu trasferu objektu
                        if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Paint_Mixer)
                        {
                            if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                            {
                                bool isSame = false;
                                for (int d = 0; d < field->belt_next.at(0)->paint.size(); d++)
                                {
                                    isSame = (isSame || (field->belt_next.at(0)->paint.at(d)->Paint_num == field->belt_object_color));
                                }
                                if (!isSame)
                                    field->belt_pos += time_pass * 7.5F;//0.016F
                                else
                                    field->belt_next.at(0)->Targeted.clear();
                            }
                        }
                        else
                            if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Painter)
                            {
                                if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                    (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                    (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                                {
                                    if (field->belt_next.at(0)->paint.size() == 0)
                                        field->belt_pos += time_pass * 7.5F;//0.016F
                                }
                                else
                                {
                                    if (field->belt_next.at(0)->Paint_objtype == CSixTree::Field::_sub_type::None)
                                        field->belt_pos += time_pass * 7.5F;//0.016F
                                }
                            }
                            else
                                field->belt_pos += time_pass * 7.5F;//0.016F
                        if ((field->belt_next.at(0)->belt_pos < field->belt_pos) && (field->belt_next.at(0)->belt_objtype != CSixTree::Field::_sub_type::None))
                            field->belt_pos = field->belt_next.at(0)->belt_pos;
                        if (field->belt_pos >= 1.0f)
                        {
                            field->belt_pos = 1.0F;
                            if (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None)
                            {
                                if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Paint_Mixer)
                                {
                                    if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                        (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                        (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                                    {
                                        if (field->belt_next.at(0)->paint.size() < 2)
                                        {
                                            bool isSame = false;
                                            for (int d = 0; d < field->belt_next.at(0)->paint.size(); d++)
                                            {
                                                isSame = isSame || (field->belt_next.at(0)->paint.at(d)->Paint_num == field->belt_object_color);
                                            }
                                            if (!isSame)
                                            {
                                                CSixTree::Field::Paint* p = new CSixTree::Field::Paint;
                                                p->Paint_num = field->belt_object_color;
                                                p->count = 1;
                                                field->belt_next.at(0)->paint.push_back(p);
                                                field->belt_pos = 0;
                                                field->belt_next.at(0)->belt_pos = 0;
                                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                                field->belt_next.at(0)->Targeted.clear();
                                            }
                                        }
                                    }
                                }
                                else
                                    if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Painter)
                                    {
                                        if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                            (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                            (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                                        {
                                            if (field->belt_next.at(0)->paint.size() == 0)
                                            {
                                                CSixTree::Field::Paint* p = new CSixTree::Field::Paint;
                                                p->Paint_num = field->belt_object_color;
                                                p->count = 1;
                                                field->belt_next.at(0)->paint.push_back(p);
                                                field->belt_pos = 0;
                                                field->belt_next.at(0)->belt_pos = 0;
                                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                                field->belt_next.at(0)->Targeted.clear();
                                            }
                                        }
                                        else
                                        {
                                            if (field->belt_next.at(0)->Paint_objtype == CSixTree::Field::_sub_type::None)
                                            {
                                                field->belt_next.at(0)->belt_object_color = field->belt_object_color;
                                                field->belt_next.at(0)->Paint_objtype = field->belt_objtype;
                                                field->belt_pos = 0;
                                                field->belt_next.at(0)->belt_pos = 0;
                                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                                field->belt_next.at(0)->Targeted.clear();
                                            }
                                        }
                                    }
                                    else
                                    {
                                        if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                        {
                                            if ((misja.object_color == field->belt_object_color) && (misja.object_type == field->belt_objtype))
                                            {
                                                misja.object_Count--;
                                            }
                                        }
                                        field->belt_next.at(0)->belt_objtype = field->belt_objtype;
                                        field->belt_next.at(0)->belt_pos = 0;
                                        field->belt_next.at(0)->belt_object_color = field->belt_object_color;
                                        field->belt_pos = 0;
                                        field->belt_objtype = CSixTree::Field::_sub_type::None;
                                        field->belt_next.at(0)->Targeted.clear();
                                        if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                        {
                                            field->belt_next.at(0)->belt_objtype = CSixTree::Field::_sub_type::None;
                                        }
                                    }
                            }
                        }
                    }
                }
    }
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Under_belt_input);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        if (field->belt_objtype != CSixTree::Field::_sub_type::None)
            if (field->belt_next.size() > 0)
                if ((field->belt_pos <= field->belt_next.at(0)->belt_pos) || (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None))
                {
                    bool targeted_started = false;
                    for (int c = 0; c < field->belt_next.at(0)->Targeted.size(); c++)
                    {
                        if (field->belt_next.at(0)->Targeted.at(c) == field)
                            targeted_started = true;
                    }
                    if (targeted_started || (field->belt_next.at(0)->Targeted.size() == 0))
                    {
                        if ((field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Output) && (field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Painter) && (field->belt_next.at(0)->Targeted.size() == 0))
                            if (!targeted_started)
                                field->belt_next.at(0)->Targeted.push_back(field);//zaznaczenie celu trasferu objektu
                        field->belt_pos += time_pass * 7.5F;//0.016F
                        if ((field->belt_next.at(0)->belt_pos < field->belt_pos) && (field->belt_next.at(0)->belt_objtype != CSixTree::Field::_sub_type::None))
                            field->belt_pos = field->belt_next.at(0)->belt_pos;
                        if (field->belt_pos >= 1.0f)
                        {
                            field->belt_pos = 1.0F;
                            if (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None)
                            {
                                field->belt_next.at(0)->belt_objtype = field->belt_objtype;
                                field->belt_next.at(0)->belt_pos = 0;
                                field->belt_next.at(0)->belt_object_color = field->belt_object_color;
                                field->belt_pos = 0;
                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                field->belt_next.at(0)->Targeted.clear();
                                if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                {
                                    field->belt_next.at(0)->belt_objtype = CSixTree::Field::_sub_type::None;
                                }
                            }
                        }
                    }
                }
    }
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Under_belt_output);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        if (field->belt_objtype != CSixTree::Field::_sub_type::None)
            if (field->belt_next.size() > 0)
                if ((field->belt_pos <= field->belt_next.at(0)->belt_pos) || (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None))
                {
                    bool targeted_started = false;
                    for (int c = 0; c < field->belt_next.at(0)->Targeted.size(); c++)
                    {
                        if (field->belt_next.at(0)->Targeted.at(c) == field)
                            targeted_started = true;
                    }
                    if (targeted_started || (field->belt_next.at(0)->Targeted.size() == 0))
                    {
                        if ((field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Output) && (field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Painter) && (field->belt_next.at(0)->Targeted.size() == 0))
                            if (!targeted_started)
                                field->belt_next.at(0)->Targeted.push_back(field);//zaznaczenie celu trasferu objektu
                        if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Paint_Mixer)
                        {
                            if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                            {
                                bool isSame = false;
                                for (int d = 0; d < field->belt_next.at(0)->paint.size(); d++)
                                {
                                    isSame = isSame || (field->belt_next.at(0)->paint.at(d)->Paint_num == field->belt_object_color);
                                }
                                if (!isSame)
                                    field->belt_pos += time_pass * 7.5F;//0.016F
                            }
                        }
                        else
                            if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Painter)
                            {
                                if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                    (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                    (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                                {
                                    if (field->belt_next.at(0)->paint.size() == 0)
                                        field->belt_pos += time_pass * 7.5F;//0.016F
                                }
                                else
                                {
                                    if (field->belt_next.at(0)->Paint_objtype == CSixTree::Field::_sub_type::None)
                                        field->belt_pos += time_pass * 7.5F;//0.016F
                                }
                            }
                            else
                                field->belt_pos += time_pass * 7.5F;//0.016F
                        if ((field->belt_next.at(0)->belt_pos < field->belt_pos) && (field->belt_next.at(0)->belt_objtype != CSixTree::Field::_sub_type::None))
                            field->belt_pos = field->belt_next.at(0)->belt_pos;
                        if (field->belt_pos >= 1.0f)
                        {
                            field->belt_pos = 1.0F;
                            if (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None)
                            {
                                if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Paint_Mixer)
                                {
                                    if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                        (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                        (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                                    {
                                        if (field->belt_next.at(0)->paint.size() < 2)
                                        {
                                            bool isSame = false;
                                            for (int d = 0; d < field->belt_next.at(0)->paint.size(); d++)
                                            {
                                                isSame = isSame || (field->belt_next.at(0)->paint.at(d)->Paint_num == field->belt_object_color);
                                            }
                                            if (!isSame)
                                            {
                                                CSixTree::Field::Paint* p = new CSixTree::Field::Paint;
                                                p->Paint_num = field->belt_object_color;
                                                p->count = 1;
                                                field->belt_next.at(0)->paint.push_back(p);
                                                field->belt_pos = 0;
                                                field->belt_next.at(0)->belt_pos = 0;
                                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                                field->belt_next.at(0)->Targeted.clear();
                                            }
                                        }
                                    }
                                }
                                else
                                    if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Painter)
                                    {
                                        if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                            (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                            (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                                        {
                                            if (field->belt_next.at(0)->paint.size() == 0)
                                            {
                                                CSixTree::Field::Paint* p = new CSixTree::Field::Paint;
                                                p->Paint_num = field->belt_object_color;
                                                p->count = 1;
                                                field->belt_next.at(0)->paint.push_back(p);
                                                field->belt_pos = 0;
                                                field->belt_next.at(0)->belt_pos = 0;
                                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                                field->belt_next.at(0)->Targeted.clear();
                                            }
                                        }
                                        else
                                        {
                                            if (field->belt_next.at(0)->Paint_objtype == CSixTree::Field::_sub_type::None)
                                            {
                                                field->belt_next.at(0)->belt_object_color = field->belt_object_color;
                                                field->belt_next.at(0)->Paint_objtype = field->belt_objtype;
                                                field->belt_pos = 0;
                                                field->belt_next.at(0)->belt_pos = 0;
                                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                                field->belt_next.at(0)->Targeted.clear();
                                            }
                                        }
                                    }
                                    else
                                    {
                                        if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                        {
                                            if ((misja.object_color == field->belt_object_color) && (misja.object_type == field->belt_objtype))
                                            {
                                                misja.object_Count--;
                                            }
                                        }
                                        field->belt_next.at(0)->belt_objtype = field->belt_objtype;
                                        field->belt_next.at(0)->belt_pos = 0;
                                        field->belt_next.at(0)->belt_object_color = field->belt_object_color;
                                        field->belt_pos = 0;
                                        field->belt_objtype = CSixTree::Field::_sub_type::None;
                                        field->belt_next.at(0)->Targeted.clear();
                                        if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                        {
                                            field->belt_next.at(0)->belt_objtype = CSixTree::Field::_sub_type::None;
                                        }
                                    }
                            }
                        }
                    }
                }
    }
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Spliter);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        if ((field->belt_objtype != CSixTree::Field::_sub_type::None) && (field->belt_next.size() > 0))
        {
            if (field->belt_next_pos >= field->belt_next.size())
                field->belt_next_pos = 0;
            int j = field->belt_next_pos;
            if (!((field->belt_pos <= field->belt_next.at(j)->belt_pos) || (field->belt_next.at(j)->belt_objtype == CSixTree::Field::_sub_type::None)))
            {
                field->belt_next_pos++;
                if (field->belt_next_pos >= field->belt_next.size())
                    field->belt_next_pos = 0;
            }
            if ((field->belt_pos <= field->belt_next.at(j)->belt_pos) || (field->belt_next.at(j)->belt_objtype == CSixTree::Field::_sub_type::None))
            {
                bool targeted_started = false;
                for (int c = 0; c < field->belt_next.at(j)->Targeted.size(); c++)
                {
                    if (field->belt_next.at(j)->Targeted.at(c) == field)
                        targeted_started = true;
                }
                if (targeted_started || (field->belt_next.at(j)->Targeted.size() == 0))
                {
                    if ((field->belt_next.at(j)->sub_type != CSixTree::Field::_sub_type::Output) && (field->belt_next.at(j)->sub_type != CSixTree::Field::_sub_type::Painter))
                        if (!targeted_started)
                            field->belt_next.at(j)->Targeted.push_back(field);
                    if (field->belt_next.at(j)->sub_type == CSixTree::Field::_sub_type::Paint_Mixer)
                    {
                        if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                            (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                            (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                        {
                            bool isSame = false;
                            for (int d = 0; d < field->belt_next.at(j)->paint.size(); d++)
                            {
                                isSame = isSame || (field->belt_next.at(j)->paint.at(d)->Paint_num == field->belt_object_color);
                            }
                            if (!isSame)
                                field->belt_pos += time_pass * 7.5F;//0.016F
                        }
                    }
                    else
                        if (field->belt_next.at(j)->sub_type == CSixTree::Field::_sub_type::Painter)
                        {
                            if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                            {
                                if (field->belt_next.at(j)->paint.size() == 0)
                                    field->belt_pos += time_pass * 7.5F;//0.016F
                            }
                            else
                            {
                                if (field->belt_next.at(j)->Paint_objtype == CSixTree::Field::_sub_type::None)
                                    field->belt_pos += time_pass * 7.5F;//0.016F
                            }
                        }
                        else
                            field->belt_pos += time_pass * 7.5F;//0.016F
                    if ((field->belt_next.at(j)->belt_pos < field->belt_pos) && (field->belt_next.at(j)->belt_objtype != CSixTree::Field::_sub_type::None))
                    {
                        field->belt_pos = field->belt_next.at(j)->belt_pos;
                        field->belt_next_pos++;
                        if (field->belt_next_pos >= field->belt_next.size())
                            field->belt_next_pos = 0;
                    }
                    if (field->belt_pos >= 1.0f)
                    {
                        field->belt_pos = 1.0F;
                        if (field->belt_next.at(j)->belt_objtype == CSixTree::Field::_sub_type::None)
                        {
                            if (field->belt_next.at(j)->sub_type == CSixTree::Field::_sub_type::Painter)
                            {
                                if ((field->belt_objtype == CSixTree::Field::_sub_type::Blue) ||
                                    (field->belt_objtype == CSixTree::Field::_sub_type::Red) ||
                                    (field->belt_objtype == CSixTree::Field::_sub_type::Yellow))
                                {
                                    if (field->belt_next.at(j)->paint.size() == 0)
                                    {
                                        CSixTree::Field::Paint* p = new CSixTree::Field::Paint;
                                        p->Paint_num = field->belt_object_color;
                                        p->count = 1;
                                        field->belt_next.at(j)->paint.push_back(p);
                                        field->belt_pos = 0;
                                        field->belt_next.at(j)->belt_pos = 0;
                                        field->belt_objtype = CSixTree::Field::_sub_type::None;
                                        field->belt_next.at(j)->Targeted.clear();
                                        field->belt_next_pos++;
                                        if (field->belt_next_pos >= field->belt_next.size())
                                            field->belt_next_pos = 0;
                                    }
                                }
                                else
                                {
                                    if (field->belt_next.at(j)->Paint_objtype == CSixTree::Field::_sub_type::None)
                                    {
                                        field->belt_next.at(j)->belt_object_color = field->belt_object_color;
                                        field->belt_next.at(j)->Paint_objtype = field->belt_objtype;
                                        field->belt_pos = 0;
                                        field->belt_next.at(j)->belt_pos = 0;
                                        field->belt_objtype = CSixTree::Field::_sub_type::None;
                                        field->belt_next.at(j)->Targeted.clear();
                                        field->belt_next_pos++;
                                        if (field->belt_next_pos >= field->belt_next.size())
                                            field->belt_next_pos = 0;
                                    }
                                }
                            }
                            else
                            {
                                if (field->belt_next.at(j)->sub_type == CSixTree::Field::_sub_type::Output)
                                {
                                    if ((misja.object_color == field->belt_object_color) && (misja.object_type == field->belt_objtype))
                                    {
                                        misja.object_Count--;
                                    }
                                }
                                field->belt_next.at(j)->belt_objtype = field->belt_objtype;
                                field->belt_next.at(j)->belt_pos = 0;
                                field->belt_next.at(j)->belt_object_color = field->belt_object_color;
                                field->belt_pos = 0;
                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                field->belt_next.at(j)->Targeted.clear();
                                field->belt_next_pos++;
                                if (field->belt_next_pos >= field->belt_next.size())
                                    field->belt_next_pos = 0;
                                if (field->belt_next.at(j)->sub_type == CSixTree::Field::_sub_type::Output)
                                {
                                    field->belt_next.at(j)->belt_objtype = CSixTree::Field::_sub_type::None;
                                }
                            }
                        }
                    }
                }
                else
                {
                    field->belt_next_pos++;
                    if (field->belt_next_pos >= field->belt_next.size())
                        field->belt_next_pos = 0;
                }
            }
        }
    }
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Miner);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        field->time_pass += time_pass;//0.016F
        if (field->time_pass >= 1.0f)
            field->time_pass = 1.0f;
        if (field->belt_next.size() > 0)
            if ((field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None) && (field->belt_next.at(0)->Targeted.size() == 0))
            {
                if (field->miner != 0)
                {
                    if (field->time_pass == 1.0f)
                    {
                        field->time_pass = 0;
                        field->belt_next.at(0)->belt_objtype = field->miner->sub_type;
                        switch (field->miner->sub_type)
                        {
                        case CSixTree::Field::_sub_type::Red:
                            field->belt_next.at(0)->belt_object_color = 8;
                            break;
                        case CSixTree::Field::_sub_type::Blue:
                            field->belt_next.at(0)->belt_object_color = 4;
                            break;
                        case CSixTree::Field::_sub_type::Yellow:
                            field->belt_next.at(0)->belt_object_color = 0;
                            break;
                        case CSixTree::Field::_sub_type::Square:
                            field->belt_next.at(0)->belt_object_color = -1;
                            break;
                        case CSixTree::Field::_sub_type::Triangle:
                            field->belt_next.at(0)->belt_object_color = -1;
                            break;
                        }

                        field->belt_next.at(0)->belt_pos = 0;
                    }
                }
            }
    }
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Painter);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        if (field->Paint_objtype != CSixTree::Field::_sub_type::None)
            if (field->belt_next.size() > 0)
            {
                if ((field->belt_pos <= field->belt_next.at(0)->belt_pos) || (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None))
                {
                    bool targeted_started = false;
                    for (int c = 0; c < field->belt_next.at(0)->Targeted.size(); c++)
                    {
                        if (field->belt_next.at(0)->Targeted.at(c) == field)
                            targeted_started = true;
                    }
                    if (targeted_started || (field->belt_next.at(0)->Targeted.size() == 0))
                    {
                        if ((field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Output) && (field->belt_next.at(0)->Targeted.size() == 0))
                            field->belt_next.at(0)->Targeted.push_back(field);
                        if ((field->paint.size() == 1) && (field->Paint_objtype != CSixTree::Field::_sub_type::None))
                            field->belt_pos += time_pass;//0.016F
                        if ((field->belt_next.at(0)->belt_pos < field->belt_pos) && (field->belt_next.at(0)->belt_objtype != CSixTree::Field::_sub_type::None))
                            field->belt_pos = field->belt_next.at(0)->belt_pos;
                        if (field->belt_pos >= 1.0f)
                        {
                            field->belt_pos = 1.0F;
                            if (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None)//miejce w nastepnym urzadzeniu
                            {
                                if ((field->paint.size() == 1) && (field->Paint_objtype != CSixTree::Field::_sub_type::None))
                                {
                                    if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                    {
                                        if ((misja.object_color == field->belt_object_color) && (misja.object_type == field->belt_objtype))
                                        {
                                            misja.object_Count--;
                                        }
                                    }
                                    field->belt_next.at(0)->belt_objtype = field->Paint_objtype;
                                    field->belt_next.at(0)->belt_object_color = field->paint.at(0)->Paint_num;
                                    field->belt_next.at(0)->belt_pos = 0;
                                    field->belt_pos = 0;
                                    field->belt_objtype = CSixTree::Field::_sub_type::None;
                                    field->belt_next.at(0)->Targeted.clear();
                                    delete field->paint.at(0);
                                    field->paint.clear();
                                    field->Paint_objtype = CSixTree::Field::_sub_type::None;
                                    if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                    {
                                        field->belt_next.at(0)->belt_objtype = CSixTree::Field::_sub_type::None;
                                    }
                                }
                            }
                        }
                    }
                }
            }
    }
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Paint_Mixer);
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        if (field->belt_next.size() > 0)
        {
            if ((field->belt_pos <= field->belt_next.at(0)->belt_pos) || (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None))
            {
                bool targeted_started = false;
                for (int c = 0; c < field->belt_next.at(0)->Targeted.size(); c++)
                {
                    if (field->belt_next.at(0)->Targeted.at(c) == field)
                        targeted_started = true;
                }
                if (targeted_started || (field->belt_next.at(0)->Targeted.size() == 0))
                {
                    if ((field->belt_next.at(0)->sub_type != CSixTree::Field::_sub_type::Output) && (field->belt_next.at(0)->Targeted.size() == 0))
                        field->belt_next.at(0)->Targeted.push_back(field);
                    if (field->paint.size() == 2)
                        field->belt_pos += time_pass;//0.016F
                    if ((field->belt_next.at(0)->belt_pos < field->belt_pos) && (field->belt_next.at(0)->belt_objtype != CSixTree::Field::_sub_type::None))
                        field->belt_pos = field->belt_next.at(0)->belt_pos;
                    if (field->belt_pos >= 1.0f)
                    {
                        field->belt_pos = 1.0F;
                        if (field->belt_next.at(0)->belt_objtype == CSixTree::Field::_sub_type::None)//miejce w nastepnym urzadzeniu
                        {
                            if (field->paint.size() == 2)
                            {
                                if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                {
                                    if ((misja.object_color == field->belt_object_color) && (misja.object_type == field->belt_objtype))
                                    {
                                        misja.object_Count--;
                                    }
                                }
                                field->belt_next.at(0)->belt_objtype = CSixTree::Field::_sub_type::Blue;
                                int farba_a = field->paint.at(0)->Paint_num;
                                int farba_b = field->paint.at(1)->Paint_num;
                                int faraba_wynik = 0;
                                int Farba_sub_space1 = abs(farba_a - farba_b);//10, 11 1
                                int Farba_sub_space2 = abs((max(farba_a, farba_b) - 12) - min(farba_a, farba_b));//2 ,11 1
                                if (Farba_sub_space1 < Farba_sub_space2)
                                {
                                    faraba_wynik = min(farba_a, farba_b) + Farba_sub_space1 / 2;
                                    if (faraba_wynik > 11)
                                        faraba_wynik -= 12;
                                }
                                else
                                {
                                    faraba_wynik = (max(farba_a, farba_b) - 12) + Farba_sub_space2 / 2;
                                    if (faraba_wynik < 0)
                                        faraba_wynik += 12;
                                }
                                field->belt_object_color = faraba_wynik;
                                field->belt_next.at(0)->belt_object_color = faraba_wynik;
                                field->belt_next.at(0)->belt_pos = 0;
                                field->belt_pos = 0;
                                field->belt_objtype = CSixTree::Field::_sub_type::None;
                                field->belt_next.at(0)->Targeted.clear();
                                delete field->paint.at(0);
                                delete field->paint.at(1);
                                field->paint.clear();
                                field->Paint_objtype = CSixTree::Field::_sub_type::None;
                                if (field->belt_next.at(0)->sub_type == CSixTree::Field::_sub_type::Output)
                                {
                                    field->belt_next.at(0)->belt_objtype = CSixTree::Field::_sub_type::None;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    temp = tree->node->GetFields(CSixTree::Field::_sub_type::Output);//baza
    for (size_t i = 0; i < temp->size(); i++)
    {
        CSixTree::Field* field = temp->at(i);
        x = (field->posx + 1) - (int)((posX * (float)resx) / (field_zoom));
        y = (field->posy + 1) - (int)((posY * (float)resy) / (field_zoom));
        a = ((float)x) / w;
        b = ((float)y) / e;

        object_X = a + c + real_posX;
        object_Y = b + v + real_posY;
        object_sX = fx * field->sizex;
        object_sY = fy * field->sizey;
        object_czoom = c * zoom;
        object_vzoom = v * zoom;
        field->belt_objtype = CSixTree::Field::_sub_type::None;
        field->belt_pos = 0;

        field->Targeted.clear();
        _r = connect->ColorTable[misja.object_color].r;
        _g = connect->ColorTable[misja.object_color].g;
        _b = connect->ColorTable[misja.object_color].b;
        //0-up 1 right 2 bottom 3 right
        switch (misja.object_type)
        {
        case CSixTree::Field::_sub_type::Square:
            DrawBox(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
            break;
        case CSixTree::Field::_sub_type::Triangle:
            DrawTriangle(object_X + object_czoom, object_Y + object_vzoom, fx - object_czoom * 4, fy - object_vzoom * 4, _r, _g, _b);
            break;
        }
        char str[5];
        sprintf_s(str, "%i", misja.lvl);
        font->RederText(str, object_X, object_Y-object_sY / 3.2F, object_sX*0.18F, object_sY/3.4F, object_czoom*4000, 1, 1, 1);
        sprintf_s(str, "%i", misja.object_Count);
        font->RederText(str, object_X, object_Y + object_sY / 3.2F, object_sX * 0.18F, object_sY / 3.4F, object_czoom*4000, 1, 1, 1);
    }
    //update tasks
    if (misja.object_Count <= 0)//misja wykonana
    {
        misja.lvl++;
        misja.object_Count = misja.lvl * 10;
        float object_rand = tree->GetRandom();
        if (object_rand < 0.5F)
            misja.object_type = CSixTree::Field::_sub_type::Triangle;
        else
            misja.object_type = CSixTree::Field::_sub_type::Square;
        if (misja.lvl > 10)
        {
            float color_rand = tree->GetRandom() * 11;
            if ((int)color_rand == misja.object_color)
                color_rand = 11;
            misja.object_color = color_rand;
        }
        else
        {
            int color_rand = (int)(tree->GetRandom() * 3);
            if (color_rand == 0)
                misja.object_color = 0;//YELLOW
            if (color_rand == 1)
                misja.object_color = 4;//BLUE
            if (color_rand == 2)
                misja.object_color = 8;//RED
        }
    }
    //inne budynki podglad
    c = _1_resX;
    v = _1_resY;
    x = (int)xx;
    if (xx < 0)
        x = -1;
    y = (int)yy;
    if (yy < 0)
        y = -1;
    a = ((float)x) / w;
    b = ((float)y) / e;
    object_X = a + c + posX;
    object_Y = b + v + posY;
    object_sX = fx - c;
    object_sY = fy - v;
    object_czoom = c * zoom;
    object_vzoom = v * zoom;
    xx = (xPos - posX * (float)resx) / (field_zoom);
    yy = (yPos - posY * (float)resy) / (field_zoom);

    //kolizje
    int count = 0;
    for (int x = -1; x <= w + 1; x++)
        for (int y = -1; y <= e + 1; y++)
        {
            float pa, pb;
            pa = ((float)x);
            pb = ((float)y);
            pa /= w;
            pb /= e;

            if (DoIntersect(pa + posX, pb + posY, fx, fy, xmysz, ymysz))
            {
                menu->object_posx = (x - (int)(float)((real_posX * resx) / (field_zoom)));
                menu->object_posy = (y - (int)(float)((real_posY * resy) / (field_zoom)));
                count++;
                x = 1000;
                y = 1000;
            }
        }

    bool isException = false;
    if (Selected_all != 0)
    {
        if (Selected_all->type == CSixTree::Field::_type::Resource)
            isException = true;
    }
    int kiers;
    if ((Selected_all == 0) || (isException))
        switch (menu->button_actual_select)
        {
        case 0:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = _g = _b = 0.9f;
            if ((menu->kierunek == CSixTree::Field::_kierunek::right) || (menu->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((menu->kierunek == CSixTree::Field::_kierunek::up) || (menu->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            DrawArrow(menu->kierunek);
            break;
        case 4:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = _g = _b = 0.9f;
            if ((menu->kierunek == CSixTree::Field::_kierunek::right) || (menu->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((menu->kierunek == CSixTree::Field::_kierunek::up) || (menu->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            _r = _g = 0.7f;
            _b = 0.5F;
            if (menu->kierunek == CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y, object_sX, object_sY / 2, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::right)
                DrawBox(object_X + object_sX / 2, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y + object_sY / 2, object_sX, object_sY / 2, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::left)
                DrawBox(object_X, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            DrawArrow(menu->kierunek);
            break;
        case 5://Paint_Mixer
            _r = _b = _g = 0.1f;
            if ((menu->kierunek == CSixTree::Field::_kierunek::right) || (menu->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawTriangleFull(object_X, object_Y, object_X + object_sX / 2, object_Y, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y, object_X + object_sX, object_Y, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X, object_Y + object_sY, object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((menu->kierunek == CSixTree::Field::_kierunek::up) || (menu->kierunek == CSixTree::Field::_kierunek::bottom))
            {
                DrawTriangleFull(object_X, object_Y, object_X, object_Y + object_sY / 2, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X, object_Y + object_sY / 2, object_X, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX, object_Y, object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            if (menu->kierunek == CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::right)
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::left)
                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
            DrawArrow(menu->kierunek);
            break;
        case 35:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = _g = _b = 0.9f;
            if ((menu->kierunek == CSixTree::Field::_kierunek::right) || (menu->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((menu->kierunek == CSixTree::Field::_kierunek::up) || (menu->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            _r = _g = 0.7f;
            _b = 0.5F;
            if (menu->kierunek == CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y, object_sX, object_sY / 2, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::left)
                DrawBox(object_X + object_sX / 2, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y + object_sY / 2, object_sX, object_sY / 2, _r, _g, _b);
            if (menu->kierunek == CSixTree::Field::_kierunek::right)
                DrawBox(object_X, object_Y, object_sX / 2, object_sY, _r, _g, _b);
            DrawArrow(menu->kierunek);
            break;
        case 3:
            _r = _g = _b = 0.2f;
            DrawBox(object_X, object_Y, object_sX, object_sY, _r, _g, _b);
            _r = _b = 0.75f;
            _g = 0.75f;
            if (menu->kierunek != CSixTree::Field::_kierunek::bottom)
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);//up
            if (menu->kierunek != CSixTree::Field::_kierunek::right)
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);//left
            if (menu->kierunek != CSixTree::Field::_kierunek::up)
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);//down
            if (menu->kierunek != CSixTree::Field::_kierunek::left)
                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);//prawy
            DrawArrow(menu->kierunek);
            kiers = menu->kierunek;
            kiers++;
            if (kiers > 3)
                kiers = 0;
            DrawArrow(CSixTree::Field::_kierunek(kiers));
            kiers = menu->kierunek;
            kiers--;
            if (kiers < 0)
                kiers = 3;
            DrawArrow(CSixTree::Field::_kierunek(kiers));

            break;
        case 2://painter
            _r = _b = _g = 0.1f;
            if ((menu->kierunek == CSixTree::Field::_kierunek::right) || (menu->kierunek == CSixTree::Field::_kierunek::left))
            {
                DrawBox(object_X, object_Y, object_sX, object_vzoom * 2, _r, _g, _b);
                DrawBox(object_X, object_Y + object_sY, object_sX, -object_vzoom * 2, _r, _g, _b);
            }
            if ((menu->kierunek == CSixTree::Field::_kierunek::up) || (menu->kierunek == CSixTree::Field::_kierunek::bottom))
            {

                DrawBox(object_X + object_sX, object_Y, -object_czoom * 2, object_sY, _r, _g, _b);
                DrawBox(object_X, object_Y, object_czoom * 2, object_sY, _r, _g, _b);
            }
            if (menu->kierunek == CSixTree::Field::_kierunek::up)//ok
            {
                DrawTriangleFull(object_X, object_Y, object_X, object_Y + object_sY / 2, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X, object_Y + object_sY / 2, object_X, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
            }
            if (menu->kierunek == CSixTree::Field::_kierunek::right)//ok
            {
                DrawTriangleFull(object_X, object_Y, object_X + object_sX / 2, object_Y, object_X + object_sX / 4, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y, object_X + object_sX, object_Y, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
            }
            if (menu->kierunek == CSixTree::Field::_kierunek::bottom)
            {
                DrawTriangleFull(object_X + object_sX, object_Y, object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX, object_Y + object_sY / 2, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
            }
            if (menu->kierunek == CSixTree::Field::_kierunek::left)
            {
                DrawTriangleFull(object_X, object_Y + object_sY, object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX / 4, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
                DrawTriangleFull(object_X + object_sX / 2, object_Y + object_sY, object_X + object_sX, object_Y + object_sY, object_X + object_sX / 4 + object_sX / 2, object_Y + object_sY / 4 + object_sY / 2, _r, _g, _b);
            }
            DrawArrow(menu->kierunek);
            break;
        }

    menu->Render();
    
    glEnd();
    glPopMatrix();

    SwapBuffers(hDC);
    return true;
}

/*******************
 * Enable OpenGL
 *
 *******************/

void CBase::EnableOpenGL(HWND hWnd, HDC* hDC, HGLRC* hRC)
{
    PIXELFORMATDESCRIPTOR pfd;
    int iFormat;

    /* get the device context (DC) */
    *hDC = GetDC(hWnd);

    /* set the pixel format for the DC */
    ZeroMemory(&pfd, sizeof(pfd));
    pfd.nSize = sizeof(pfd);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW |
        PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 32;
    pfd.cDepthBits = 32;
    pfd.iLayerType = PFD_MAIN_PLANE;
    iFormat = ChoosePixelFormat(*hDC, &pfd);
    SetPixelFormat(*hDC, iFormat, &pfd);

    /* create and enable the render context (RC) */
    *hRC = wglCreateContext(*hDC);
    wglMakeCurrent(*hDC, *hRC);

}


/******************
 * Disable OpenGL
 *
 ******************/
void CBase::DisableOpenGL(HWND hWnd, HDC hDC, HGLRC hRC)
{
    wglMakeCurrent(NULL, NULL);
    wglDeleteContext(hRC);
    ReleaseDC(hWnd, hDC);
}

/******************
 * draws triangle
 *
 ******************/
void CBase::DrawTriangleFull(float xa, float ya, float xb, float yb, float xc, float yc, float r, float g, float b)
{
    if (state_opengl != GL_QUADS)
    {
        glEnd();
        glBegin(GL_QUADS);
        state_opengl = GL_QUADS;
    }
    glColor3f(r, g, b);
    ya = 1 - ya; yb = 1 - yb; yc = 1 - yc;
    xa *= 2.0f; xb *= 2.0f; xc *= 2.0f;
    ya *= 2.0f; yb *= 2.0f; yc *= 2.0f;

    glVertex2f(xa - 1.0f, ya - 1.0f);
    glVertex2f(xb - 1.0f, yb - 1.0f);
    glVertex2f(xc - 1.0f, yc - 1.0f);
    glVertex2f(xc - 1.0f, yc - 1.0f);
}

void CBase::DrawArrow(CSixTree::Field::_kierunek kier)
{
    _r = _g = 0.9f;
    _b = 0.1f;
    if (kier == CSixTree::Field::_kierunek::up)
        DrawTriangleFull(object_X + object_czoom * 8, object_Y + object_sY / 4, object_X + object_sX / 2, object_Y + object_vzoom * 4, object_X + object_sX - object_czoom * 8, object_Y + object_sY / 4, _r, _g, _b);
    if (kier == CSixTree::Field::_kierunek::right)
        DrawTriangleFull(object_X + 3 * object_sX / 4, object_Y + object_vzoom * 8, object_X + object_sX - object_czoom * 4, object_Y + object_sY / 2, object_X + 3 * object_sX / 4, object_Y + object_sY - object_vzoom * 8, _r, _g, _b);
    if (kier == CSixTree::Field::_kierunek::bottom)
        DrawTriangleFull(object_X + object_czoom * 8, object_Y + 3 * object_sY / 4, object_X + object_sX / 2, object_Y + object_sY - object_vzoom * 4, object_X + object_sX - object_czoom * 8, object_Y + 3 * object_sY / 4, _r, _g, _b);
    if (kier == CSixTree::Field::_kierunek::left)
        DrawTriangleFull(object_X + object_sX / 4, object_Y + object_vzoom * 8, object_X + object_czoom * 4, object_Y + object_sY / 2, object_X + object_sX / 4, object_Y + object_sY - object_vzoom * 8, _r, _g, _b);

}

/******************
 * draws triangle
 *
 ******************/
void CBase::DrawTriangle(float x, float y, float sx, float sy, float r, float g, float b)
{
    if (state_opengl != GL_QUADS)
    {
        glEnd();
        glBegin(GL_QUADS);
        state_opengl = GL_QUADS;
    }
    glColor3f(r, g, b);
    y = 1 - y;
    x *= 2.0f;
    y *= 2.0f;
    sx *= 2.0f;
    sy *= 2.0f;
    glVertex2f((x + sx / 2) - 1.0f, y - 1.0f);
    glVertex2f((x + sx / 2) - 1.0f, y - 1.0f);
    glVertex2f((x + sx) - 1.0f, (y - sy) - 1.0f);
    glVertex2f(x - 1.0f, (y - sy) - 1.0f);
}

/******************
 * draws circle
 *
 ******************/
void CBase::DrawCircle(float x, float y, float fi1, float fi2, float r, float g, float b)
{
    if (state_opengl != GL_QUADS)
    {
        glEnd();
        glBegin(GL_QUADS);
        state_opengl = GL_QUADS;
    }
    //sinus
    glColor3f(r, g, b);
    y = 1 - y;
    x *= 2.0f;
    y *= 2.0f;
    fi1 *= 2.0f;
    fi2 *= 2.0f;

    for (int i = 0; i < 48; i += 2)
    {
        float xa = x + sinus[i] * fi1;
        float ya = y + cosinus[i] * fi2;
        float xb = x + sinus[i + 1] * fi1;
        float yb = y + cosinus[i + 1] * fi2;
        float xc = x + sinus[i + 2] * fi1;
        float yc = y + cosinus[i + 2] * fi2;

        glVertex2f((x)-1.0f, (y)-1.0f);
        glVertex2f((xa)-1.0f, (ya)-1.0f);
        glVertex2f((xb)-1.0f, (yb)-1.0f);
        glVertex2f((xc)-1.0f, (yc)-1.0f);
    }
}

/******************
 * draws line
 *
 ******************/
void CBase::DrawLine(float x, float y, float sx, float sy, float fi, float r, float g, float b)
{
    if ((line_with != fi) || (state_opengl != GL_LINES))
    {
        glEnd();
        glLineWidth(fi);
        glBegin(GL_LINES);
        state_opengl = GL_LINES;
        line_with = fi;
    }
    glColor3f(r, g, b);

    y = 1 - y;
    sy = 1 - sy;
    x *= 2.0f;
    y *= 2.0f;
    sx *= 2.0f;
    sy *= 2.0f;

    glVertex2f(x - 1.0f, y - 1.0f);
    glVertex2f(sx - 1.0f, sy - 1.0f);
}

/******************
 * draws box
 *
 ******************/
void CBase::DrawBox(float x, float y, float sx, float sy, float r, float g, float b)
{
    if (state_opengl != GL_QUADS)
    {
        glEnd();
        glBegin(GL_QUADS);
        state_opengl = GL_QUADS;
    }
    glColor3f(r, g, b);
    y = 1 - y;
    x *= 2.0f;
    y *= 2.0f;
    sx *= 2.0f;
    sy *= 2.0f;
    glVertex2f(x - 1.0f, y - 1.0f);
    glVertex2f((x + sx) - 1.0f, y - 1.0f);
    glVertex2f((x + sx) - 1.0f, (y - sy) - 1.0f);
    glVertex2f(x - 1.0f, (y - sy) - 1.0f);
}
//mouse support
//WM_MOUSEFIRST | WM_MOUSEMOVE | WM_LBUTTONDOWN | WM_LBUTTONUP
//WM_LBUTTONDBLCLK | WM_RBUTTONDOWN | WM_RBUTTONUP | WM_RBUTTONDBLCLK
//WM_MBUTTONDOWN | WM_MBUTTONUP | WM_MBUTTONDBLCLK | WM_MOUSEWHEEL
//WM_XBUTTONDOWN | WM_XBUTTONUP | WM_XBUTTONDBLCLK | WM_MOUSEHWHEEL
//WM_MOUSELAST
bool CBase::MouseKey(UINT message, WPARAM wParam, WPARAM lParam)
{
    short fwKeys = GET_KEYSTATE_WPARAM(wParam);
    zDelta = GET_WHEEL_DELTA_WPARAM(wParam);
    xPos = GET_X_LPARAM(lParam);
    yPos = GET_Y_LPARAM(lParam);

    control = (fwKeys == MK_CONTROL);
    shift = (fwKeys == MK_SHIFT);
    if (message == WM_LBUTTONDOWN)
        button_left = true;
    if (message == WM_LBUTTONUP)
        button_left = false;
    if (message == WM_MBUTTONDOWN)
        button_midle = true;
    if (message == WM_MBUTTONUP)
        button_midle = false;
    if (message == WM_RBUTTONDOWN)
        button_right = true;
    if (message == WM_RBUTTONUP)
        button_right = false;
    auto old_zoom = zoom;
    if (zDelta < 0)
        zoom /= 1.2f;
    if (zDelta > 0)
        zoom *= 1.2f;
    if (zoom > 20)
        zoom = 20;
    if (zoom < 0.2F)
        zoom = 0.2F;
    if (zDelta != 0)
    {
        float w, e;
        w = (float)resx;
        w /= Field * zoom;
        e = (float)resy;
        e /= Field * zoom;
        float fx, fy;
        fx = (1.0f / w);
        fy = (1.0f / e);

        float x = ((real_posX - (xPos / (float)resx)) / old_zoom);
        float y = ((real_posY - (yPos / (float)resy)) / old_zoom);
        real_posX = (x * zoom) + (xPos / (float)resx);
        real_posY = (y * zoom) + (yPos / (float)resy);

        posX = real_posX;
        posY = real_posY;

        while (posX > fx)
            posX -= fx;
        while (posX <= -fx)
            posX += fx;
        while (posY > fy)
            posY -= fy;
        while (posY <= -fy)
            posY += fy;


    }
    if (button_right)
    {
        if (message == WM_RBUTTONDOWN)
        {
            old_xPos = xPos;
            old_yPos = yPos;
        }
        else
        {
            short tempx = old_xPos - xPos;
            short tempy = old_yPos - yPos;
            old_xPos = xPos;
            old_yPos = yPos;
            float w, e;
            w = (float)resx;
            w /= Field * zoom;
            e = (float)resy;
            e /= Field * zoom;
            float fx, fy;
            fx = (1.0f / w);
            fy = (1.0f / e);
            real_posX -= ((float)tempx) / (float)resx;
            real_posY -= ((float)tempy) / (float)resy;
            posX = real_posX;
            posY = real_posY;

            while (posX > fx)
                posX -= fx;
            while (posX <= -fx)
                posX += fx;
            while (posY > fy)
                posY -= fy;
            while (posY <= -fy)
                posY += fy;
        }
    }
    return menu->MouseKey(message, wParam, lParam);//true means quit game
}

bool CBase::PressKeyAsync()
{
    if (!isInicjalized)
        return false;
    DWORD val = GetTickCount64() - timer2;
    float time_pass = ((float)val) / 1000.0F;
    timer2 = GetTickCount64();
    float move = (0.02f / zoom) * time_pass * 20;
    if (GetAsyncKeyState('W'))
    {
        posY += move;
        real_posY += move;
    }
    if (GetAsyncKeyState('S'))
    {
        posY -= move;
        real_posY -= move;
    }
    if (GetAsyncKeyState('A'))
    {
        posX += move;
        real_posX += move;
    }
    if (GetAsyncKeyState('D'))
    {
        posX -= move;
        real_posX -= move;
    }
    if (GetAsyncKeyState('Q') & 1)
    {
        if (Selected_all != 0)
        {
            menu->button_actual_select = -1;
            switch (Selected_all->sub_type)
            {
            case CSixTree::Field::_sub_type::Belt:
                menu->button_actual_select = 0;
                break;
            case CSixTree::Field::_sub_type::Miner:
                menu->button_actual_select = 1;
                break;
            case CSixTree::Field::_sub_type::Spliter:
                menu->button_actual_select = 3;
                break;
            case CSixTree::Field::_sub_type::Painter:
                menu->button_actual_select = 2;
                break;
            case CSixTree::Field::_sub_type::Under_belt_input:
                menu->button_actual_select = 4;
                break;
            case CSixTree::Field::_sub_type::Paint_Mixer:
                menu->button_actual_select = 5;
                break;
            case CSixTree::Field::_sub_type::Under_belt_output:
                menu->button_actual_select = 35;
                break;
            }

            switch (Selected_all->type)
            {
            case CSixTree::Field::_type::Resource:
                menu->button_actual_select = 1;
                break;
            default:
                menu->kierunek = Selected_all->kierunek;
            }

        }
        else
            menu->button_actual_select = -1;//no object lose object
    }
    if ((GetAsyncKeyState(VK_DELETE) & 1) && (Selected_all != 0))
    {
        if ((Selected_all->type == CSixTree::Field::_type::Bulding) && (Selected_all->sub_type != CSixTree::Field::_sub_type::Output))
        {
            connect->RemoveObject(Selected_all);
            Selected_all = 0;
        }
    }
    if (GetAsyncKeyState('R') & 1)
    {
        if (Selected_all == 0)
        {
            CSixTree::Field::_kierunek k = CSixTree::Field::_kierunek::up;
            switch (menu->kierunek)
            {
            case CSixTree::Field::_kierunek::up:
                k = CSixTree::Field::_kierunek::right;
                break;
            case CSixTree::Field::_kierunek::right:
                k = CSixTree::Field::_kierunek::bottom;
                break;
            case CSixTree::Field::_kierunek::bottom:
                k = CSixTree::Field::_kierunek::left;
                break;
            case CSixTree::Field::_kierunek::left:
                k = CSixTree::Field::_kierunek::up;
                break;
            }
            menu->kierunek = k;
        }
        if (Selected_all != 0)
        {
            if (Selected_all->type == CSixTree::Field::_type::Resource)
            {
                CSixTree::Field::_kierunek k = CSixTree::Field::_kierunek::up;
                switch (menu->kierunek)
                {
                case CSixTree::Field::_kierunek::up:
                    k = CSixTree::Field::_kierunek::right;
                    break;
                case CSixTree::Field::_kierunek::right:
                    k = CSixTree::Field::_kierunek::bottom;
                    break;
                case CSixTree::Field::_kierunek::bottom:
                    k = CSixTree::Field::_kierunek::left;
                    break;
                case CSixTree::Field::_kierunek::left:
                    k = CSixTree::Field::_kierunek::up;
                    break;
                }
                menu->kierunek = k;

            }
            else
            {
                Selected_all->Targeted.clear();
                CSixTree::Field::_kierunek k;
                switch (Selected_all->kierunek)
                {
                case CSixTree::Field::_kierunek::up:
                    k = CSixTree::Field::_kierunek::right;
                    break;
                case CSixTree::Field::_kierunek::right:
                    k = CSixTree::Field::_kierunek::bottom;
                    break;
                case CSixTree::Field::_kierunek::bottom:
                    k = CSixTree::Field::_kierunek::left;
                    break;
                case CSixTree::Field::_kierunek::left:
                    k = CSixTree::Field::_kierunek::up;
                    break;
                }
                Selected_all->kierunek = k;
                menu->kierunek = Selected_all->kierunek;
                connect->DisconnectObject(Selected_all);
                connect->ConnectObject(Selected_all);
            }
        }
    }
    float w, e;
    w = (float)resx;
    w /= Field * zoom;
    e = (float)resy;
    e /= Field * zoom;

    float fx, fy;
    fx = (1.0f / w);
    fy = (1.0f / e);


    posX = real_posX;
    posY = real_posY;

    while (posX > fx)
        posX -= fx;
    while (posX < -fx)
        posX += fx;
    while (posY > fy)
        posY -= fy;
    while (posY < -fy)
        posY += fy;
    return true;
}

//Keyboard
bool CBase::PressedKey(WPARAM wParam)
{
    if (!isInicjalized)
        return false;
    if (wParam == VK_ESCAPE)
        return true;
    return menu->PressedKey(wParam);//true means quit game
}

