#include <vector>
using namespace std;
#include "CSixTree.h"

constexpr auto Sector_size = 10;

CSixTree::CSixTree()
{
	last = true;
	size = Sector_size;
	posx = 0;
	posy = 0;
	for (int i = 0; i < 9; i++)
		tree[i] = nullptr;
}

CSixTree::CSixTree(int size, int tx, int ty)
{
	posx = tx;
	posy = ty;
	if (size > Sector_size)
	{
		last = false;
		for (int i = 0; i < 9; i++)
		{
			int _tx, _ty;
			if ((i == 0) || (i == 3) || (i == 6))
				_tx = tx - (size / 3);
			if ((i == 1) || (i == 4) || (i == 7))
				_tx = tx;
			if ((i == 2) || (i == 5) || (i == 8))
				_tx = tx + (size / 3);
			if ((i == 0) || (i == 1) || (i == 2))
				_ty = ty - (size / 3);
			if ((i == 3) || (i == 4) || (i == 5))
				_ty = ty;
			if ((i == 6) || (i == 7) || (i == 8))
				_ty = ty + (size / 3);
			tree[i] = new CSixTree(size / 3, _tx, _ty);
			// 0 1 2
			// 3 4 5
			// 6 7 8
		}
	}
	else
	{
		last = true;
		for (int i = 0; i < 9; i++)
			tree[i] = nullptr;
	}
	this->size = size;
}

void CSixTree::RemoveField(Field* field)
{
	if (last == true)
	{
		for (int i = 0; i < fields.size(); i++)
		{
			if (field == fields.at(i))
			{
				if (fields.size() == 1)
					fields.clear();
				else
					fields.erase(fields.begin() + i);
				delete field;
				return;
			}
		}
	}
	else
	{
		for (int i = 0; i < 9; i++)
		{
			if (tree[i]->TestCords(field->posx, field->posy))
			{
				tree[i]->RemoveField(field);
				return;//found target
			}
		}
	}
}

bool CSixTree::TestCords(int x, int y)
{
	return ((x >= posx - size / 2) && (y >= posy - size / 2) && (x <= posx + size / 2) && (y <= posy + size / 2));
}

CSixTree::CSixTree(CSixTree* first)
{
	posx = posy = 0;
	last = false;
	tree[4] = first;
	size = first->size * 3;
	for (int i = 0; i < 9; i++)
		if (i != 4)
		{
			int _tx, _ty;
			if ((i == 0) || (i == 3) || (i == 6))
				_tx = -size / 3;
			if ((i == 1) || (i == 4) || (i == 7))
				_tx = 0;
			if ((i == 2) || (i == 5) || (i == 8))
				_tx = size / 3;
			if ((i == 0) || (i == 1) || (i == 2))
				_ty = -size / 3;
			if ((i == 3) || (i == 4) || (i == 5))
				_ty = 0;
			if ((i == 6) || (i == 7) || (i == 8))
				_ty = size / 3;
			tree[i] = new CSixTree(first->size, _tx, _ty);
		}
}

CSixTree::~CSixTree()
{
	for (int i = 0; i < 9; i++)
		if (tree[i] != nullptr)
			delete tree[i];
	for (unsigned int i = 0; i < fields.size(); i++)
	{
		//delete fields[i];
	}
	fields.clear();
}

vector<CSixTree::Field*>* CSixTree::GetFields(Field::_sub_type type)
{
	field.clear();
	if (last == true)
	{
		for (unsigned int i = 0; i < fields.size(); i++)
		{
			if (fields[i]->sub_type == type)
				field.push_back(fields[i]);
		}
	}
	else
	{
		for (int i = 0; i < 9; i++)
		{
			vector<CSixTree::Field*>* temp = tree[i]->GetFields(type);
			for (unsigned int i = 0; i < temp->size(); i++)
				field.push_back(temp->at(i));
		}
	}
	return &field;
}

void CSixTree::AddField(Field* field)
{
	if (last == true)
	{
		fields.push_back(field);
	}
	else
	{
		for (int i = 0; i < 9; i++)
		{
			if (tree[i]->TestCords(field->posx, field->posy))
			{
				tree[i]->AddField(field);
				return;//found target
			}
		}
	}
}

vector<CSixTree::Field*>* CSixTree::GetArea(int x, int y, int sizex, int sizey)
{
	field.clear();
	if (last == true)
	{
		for (unsigned int i = 0; i < fields.size(); i++)
			field.push_back(fields[i]);
	}
	else
	{
		for (int i = 0; i < 9; i++)
		{
			if (DoBoxesIntersect(tree[i], x, y, sizex, sizey))
			{
				vector<CSixTree::Field*>* temp = tree[i]->GetArea(x, y, sizex, sizey);
				for (unsigned int i = 0; i < temp->size(); i++)
					field.push_back(temp->at(i));
			}
		}
	}
	return &field;
}
