#include <vector>
using namespace std;
#include "CSixTree.h"
#include "CmapGen.h"
#include <stdlib.h>
#include <time.h>

constexpr auto Resource_Distance = 50;//resource distance
constexpr auto Resource_Size_Min = 5;
constexpr auto Resource_Size_Max = 20;
constexpr auto Resource_Position_Variation = 40;
constexpr auto Resource_Count_Min = 2000;
constexpr auto Resource_Count_Max = 5000;
constexpr auto First_Area = 500;


CmapGen::CmapGen()
{
	node = new CSixTree();
	srand((unsigned int)time(NULL));

	Boundaries.top = -First_Area / 2;
	Boundaries.bottom = First_Area / 2;
	Boundaries.left = -First_Area / 2;
	Boundaries.right = First_Area / 2;

	Count.top = Boundaries.top / Resource_Distance;
	Count.bottom = Boundaries.bottom / Resource_Distance;
	Count.left = Boundaries.left / Resource_Distance;
	Count.right = Boundaries.right / Resource_Distance;
}

CmapGen::~CmapGen()
{
	delete node;
}

float CmapGen::GetRandom()//0...1
{
	auto random = rand() % 10000;
	return ((float)random) / 10000.0F;
}

bool CmapGen::CreateMap()
{
	//add base:
	CSixTree::Field* field = new CSixTree::Field;
	field->posx = field->posy = 0;
	field->sizex = 3;
	field->sizey = 3;
	field->Count = 0;
	field->kierunek = CSixTree::Field::_kierunek::up;
	field->type = CSixTree::Field::_type::Bulding;
	field->sub_type = CSixTree::Field::_sub_type::Output;
	node->AddField(field);
	old_positions.push_back(Vector2D(0,0));

	auto Count = First_Area / Resource_Distance;
	for (auto x = 0; x <= Count; x++)
	{
		for (auto y = 0; y <= Count; y++)
		{
			if (!((x == Count / 2) && (y == Count / 2)))
			{
				auto _x = -First_Area / 2 + x * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto _y = -First_Area / 2 + y * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto resource_Count = Resource_Size_Min + GetRandom() * (Resource_Size_Max - Resource_Size_Min);
				auto resource_type = GetRandom() * 5;
				auto resource_Axis = sqrt(resource_Count);
				auto actual = 0;
				for (auto px = 0; px < resource_Axis; px++)
				{
					for (auto py = 0; py < resource_Axis; py++)
					{
						if (actual < resource_Count)
						{
							actual++;
							auto Resource_Amound = Resource_Count_Min + GetRandom() * (Resource_Count_Max - Resource_Count_Min);
							CSixTree::Field* field = new CSixTree::Field;
							field->posx = (_x - resource_Axis / 2) + px;
							field->posy = (_y - resource_Axis / 2) + py;
							field->Count = (int)Resource_Amound;
							field->type = CSixTree::Field::_type::Resource;
							field->sub_type = CSixTree::Field::_sub_type((int)resource_type);
							if (node->TestCords(field->posx, field->posy))
								node->AddField(field);
							else
							{
								while (!node->TestCords(field->posx, field->posy))
									node = new CSixTree(node);
								node->AddField(field);
							}
						}
					}
				}
			}
		}
	}
	return true;
}

vector<CSixTree::Field*>* CmapGen::GetArea(int x, int y, int sizex, int sizey)//left top corner position
{
	int result;
	while ((result = DoIntersect(x, y, sizex, sizey)) != 0)
	{
		//we rebuild one axis
		switch (result)
		{
		case 1://top
			for (auto x = Count.left; x <= Count.right; x++)
			{
				auto y = Count.top - 1;
				auto _x = x * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto _y = y * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto resource_Count = Resource_Size_Min + GetRandom() * (Resource_Size_Max - Resource_Size_Min);
				auto resource_type = GetRandom() * 5;
				auto resource_Axis = sqrt(resource_Count);
				auto actual = 0;
				for (auto px = 0; px < resource_Axis; px++)
				{
					for (auto py = 0; py < resource_Axis; py++)
					{
						if (actual < resource_Count)
						{
							actual++;
							auto Resource_Amound = Resource_Count_Min + GetRandom() * (Resource_Count_Max - Resource_Count_Min);
							CSixTree::Field* field = new CSixTree::Field;
							field->posx = (_x - resource_Axis / 2) + px;
							field->posy = (_y - resource_Axis / 2) + py;
							field->Count = (int)Resource_Amound;
							field->type = CSixTree::Field::_type::Resource;
							field->sub_type = CSixTree::Field::_sub_type((int)resource_type);
							if (node->TestCords(field->posx, field->posy))
								node->AddField(field);
							else
							{
								while (!node->TestCords(field->posx, field->posy))
									node = new CSixTree(node);
								node->AddField(field);
							}
						}
					}
				}
			}
			Count.top--;
			Boundaries.top -= Resource_Distance;
			break;
		case 2://bottom
			for (auto x = Count.left; x <= Count.right; x++)
			{
				auto y = Count.bottom + 1;
				auto _x = x * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto _y = y * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto resource_Count = Resource_Size_Min + GetRandom() * (Resource_Size_Max - Resource_Size_Min);
				auto resource_type = GetRandom() * 5;
				auto resource_Axis = sqrt(resource_Count);
				auto actual = 0;
				for (auto px = 0; px < resource_Axis; px++)
				{
					for (auto py = 0; py < resource_Axis; py++)
					{
						if (actual < resource_Count)
						{
							actual++;
							auto Resource_Amound = Resource_Count_Min + GetRandom() * (Resource_Count_Max - Resource_Count_Min);
							CSixTree::Field* field = new CSixTree::Field;
							field->posx = (_x - resource_Axis / 2) + px;
							field->posy = (_y - resource_Axis / 2) + py;
							field->Count = (int)Resource_Amound;
							field->type = CSixTree::Field::_type::Resource;
							field->sub_type = CSixTree::Field::_sub_type((int)resource_type);
							if (node->TestCords(field->posx, field->posy))
								node->AddField(field);
							else
							{
								while (!node->TestCords(field->posx, field->posy))
									node = new CSixTree(node);
								node->AddField(field);
							}
						}
					}
				}
			}
			Count.bottom++;
			Boundaries.bottom += Resource_Distance;
			break;
		case 3://left
			for (auto y = Count.top; y <= Count.bottom; y++)
			{
				auto x = Count.left - 1;
				auto _x = x * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto _y = y * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto resource_Count = Resource_Size_Min + GetRandom() * (Resource_Size_Max - Resource_Size_Min);
				auto resource_type = GetRandom() * 5;
				auto resource_Axis = sqrt(resource_Count);
				auto actual = 0;
				for (auto px = 0; px < resource_Axis; px++)
				{
					for (auto py = 0; py < resource_Axis; py++)
					{
						if (actual < resource_Count)
						{
							actual++;
							auto Resource_Amound = Resource_Count_Min + GetRandom() * (Resource_Count_Max - Resource_Count_Min);
							CSixTree::Field* field = new CSixTree::Field;
							field->posx = (_x - resource_Axis / 2) + px;
							field->posy = (_y - resource_Axis / 2) + py;
							field->Count = (int)Resource_Amound;
							field->type = CSixTree::Field::_type::Resource;
							field->sub_type = CSixTree::Field::_sub_type((int)resource_type);
							if (node->TestCords(field->posx, field->posy))
								node->AddField(field);
							else
							{
								while (!node->TestCords(field->posx, field->posy))
									node = new CSixTree(node);
								node->AddField(field);
							}
						}
					}
				}
			}
			Count.left--;
			Boundaries.left -= Resource_Distance;
			break;
		case 4://right
			for (auto y = Count.top; y <= Count.bottom; y++)
			{
				auto x = Count.right + 1;
				auto _x = x * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto _y = y * Resource_Distance + Resource_Position_Variation * GetRandom();
				auto resource_Count = Resource_Size_Min + GetRandom() * (Resource_Size_Max - Resource_Size_Min);
				auto resource_type = GetRandom() * 5;
				auto resource_Axis = sqrt(resource_Count);
				auto actual = 0;
				for (auto px = 0; px < resource_Axis; px++)
				{
					for (auto py = 0; py < resource_Axis; py++)
					{
						if (actual < resource_Count)
						{
							actual++;
							auto Resource_Amound = Resource_Count_Min + GetRandom() * (Resource_Count_Max - Resource_Count_Min);
							CSixTree::Field* field = new CSixTree::Field;
							field->posx = (_x - resource_Axis / 2) + px;
							field->posy = (_y - resource_Axis / 2) + py;
							field->Count = (int)Resource_Amound;
							field->type = CSixTree::Field::_type::Resource;
							field->sub_type = CSixTree::Field::_sub_type((int)resource_type);
							if (node->TestCords(field->posx, field->posy))
								node->AddField(field);
							else
							{
								while (!node->TestCords(field->posx, field->posy))
									node = new CSixTree(node);
								node->AddField(field);
							}
						}
					}
				}
			}
			Count.right++;
			Boundaries.right += Resource_Distance;
			break;
		}
	}
	return node->GetArea(x, y, sizex, sizey);
}